/**************************************************************************************************
/* This macro is used to fit joint models, two-stage models, and time-varying covariates model.   *
/*                                                                                                *
/* 1. Variables                                                                                   *
/* id: subject id;                                                                                *
/* var: longitudinal measure;                                                                     *
/* t: time at which the longitudinal measure was taken;                                           *
/* st: survival time;                                                                             *
/* ** Notice (i) t and st have the same unit;                                                     *
/*           (ii) it's better to use month in order to get better convergence;                    *
/* delta: censoring indicator (1=death, 0=censored);                                              *
/* lcv1, lcv2, ...: covariates for the longitudinal component;                                    *
/* scv1, scv2, ...: covariates for the survival component;                                        *
/*                                                                                                *
/* TS: indicates whether to implement the joint model or the correspoinding two-stage model;      *
/* TMAXI: tmax adjustment;                                                                        *
/* WEIGHT: the proportion of max(st-tmax, 0);                                                     *
/* NPIECES: number of intervals for the piecewise constant baseline hazard;                       *
/* PARTITION: partition algorithm (1=ESQP, 2=LBSQP, 3=MBSQP, 4=RBSQP);                            *
/* OPTIONS: options that are available in the PROC NLMIXED statement;                             *
/* OUTPUT: name of the output rich text file (rtf).                                               *
/*                                                                                                *
/* 2. Data sets                                                                                   *
/* LONG: data set for longtudinal measures which may have three columns in this order: id, var,   *
/*     and t, or may have more than these three columns and the rest columns are the covariates   *
/*     for the longitudinal component.                                                            *
/* SURV: data set for survival time whose columns are arranged as follows: id, st, delta, and     *
/*       the rest colums are the covariates.                                                      *
/* INITIAL: data set for the initial values of the parameters. Not required.                      *
/* ** Notice that no missing values are allowed in LONG and SURV.                                 *                                                                          *
/*                                                                                                *
/* 3. Models                                                                                      *
/* There are five models can be chosen for MODEL:                                                 *
/*  1. SPM1L (Shared parameter model 1 with linear trajectory);                                   *
/*  2. SPM1Q (Shared parameter model 1 with quadratic trajectory);                                *
/*  3. SPM2L (Shared parameter model 2 with linear trajectory);                                   *
/*  4. SPM2Q (Shared parameter model 2 with quadratic trajectory);                                *                                                                    *
/*  5. TVC (Time-varying covariates model).                                                       *
/**************************************************************************************************/

%macro JMFit(LONG, SURV, MODEL, TS, TMAXI, WEIGHT, NPIECES, PARTITION, OPTIONS, INITIAL, OUTPUT);
 %if &model=SPM1L %then %do;
   %include jmfit(SPM1L.sas);
    %SPM1L(&long, &surv, &ts, &tmaxi, &weight, &npieces, &partition, &options, &initial, &output);
 %end;

 %if &model=SPM1Q %then %do;
   %include jmfit(SPM1Q.sas);
    %SPM1Q(&long, &surv, &ts, &tmaxi, &weight, &npieces, &partition, &options, &initial, &output);
 %end;

 %if &model=SPM2L %then %do;
   %include jmfit(SPM2L.sas);
    %SPM2L(&long, &surv, &ts, &npieces, &partition, &options, &initial, &output);
 %end;

 %if &model=SPM2Q %then %do;
   %include jmfit(SPM2Q.sas);
    %SPM2Q(&long, &surv, &ts, &npieces, &partition, &options, &initial, &output);
 %end;

 %if &model=TVC %then %do;
   %include jmfit(TVC.sas);
    %TVC(&long, &surv, &npieces, &partition, &options, &initial, &output);
 %end;

%mend JMFit;




