/**************************************************************************************************
/* This macro is used to fit SPM2L.                                                               *
/*                                                                                                *
/* 1. Variables                                                                                   *
/* id: subject id;                                                                                *
/* var: longitudinal measure;                                                                     *
/* t: time at which the longitudinal measure was taken;                                           *
/* st: survival time;                                                                             *
/* ** Notice (i) t and st have the same unit;                                                     *
/*           (ii) it's better to use month in order to get better convergence;                    *
/* delta: censoring indicator (1=death, 0=censored);                                              *
/* lcv1, lcv2, ...: covariates for the longitudinal component;                                    *
/* scv1, scv2, ...: covariates for the survival component;                                        *
/*                                                                                                *
/* TS: indicates whether to implement SPM2L or the correspoinding two-stage model;                *
/* NPIECES: number of intervals for the piecewise constant baseline hazard;                       *
/* PARTITION: partition algorithm (1=ESQP, 2=LBSQP, 3=MBSQP, 4=RBSQP);                            *
/* OPTIONS: options that are available in the PROC NLMIXED statement;                             *
/* OUTPUT: name of the output rich text file (rtf).                                               *
/*                                                                                                *
/* 2. Data sets                                                                                   *
/* LONG: data set for longtudinal measures which may have three columns in this order: id, var,   *
/*     and t, or may have more than these three columns and the rest columns are the covariates   *
/*     for the longitudinal component.                                                            *
/* SURV: data set for survival time whose columns are arranged as follows: id, st, delta, and     *
/*       the rest colums are the covariates.                                                      *
/* INITIAL: data set for the initial values of the parameters in SPM2L. Not required.             *
/* ** Notice that no missing values are allowed in LONG and SURV.                                 *   
/**************************************************************************************************/

%macro SPM2L(long, surv, ts, npieces, partition, options, initial, output);
/* Data preparation */
proc contents data=&long noprint out=_long_all;
run;

proc sql noprint;
  select count(*) into :ncol_long
    from _long_all;
quit;

%if &ncol_long = 3 %then %do;
proc sql noprint;
  select NAME into :id 
     from _long_all where (VARNUM=1);
  select NAME into :var 
     from _long_all where (VARNUM=2);
  select NAME into :t 
     from _long_all where (VARNUM=3);
quit;

data _null_;
 x=0;
 call symput("nlcv", x);
run;
%end;

%if &ncol_long > 3 %then %do;
proc sql noprint;
  select NAME into :id 
    from _long_all where (VARNUM=1);
  select NAME into :var 
    from _long_all where (VARNUM=2);
  select NAME into :t 
    from _long_all where (VARNUM=3);
  select NAME into :lcvs separated by " " 
    from _long_all where (VARNUM > 3) order by VARNUM;
  select count(*) into :nlcv
    from _long_all where (VARNUM > 3);
  select NAME into :lcv1-:lcv%left(&nlcv) 
    from _long_all where (VARNUM > 3) order by VARNUM;
quit;

%end;

%let length=%length(&var);

proc sql noprint;
  select NAME format=$&length.. into :var 
     from _long_all where (VARNUM=2);
quit;

proc contents data=&surv noprint out=_surv_all;
run;

proc sql noprint;
  select NAME into :st
    from _surv_all where (VARNUM=2);
  select NAME into :delta
    from _surv_all where (VARNUM=3);
  select count(*) into :nscv 
    from _surv_all where (VARNUM > 3);
  select NAME into :scv1-:scv%left(&nscv) 
    from _surv_all where (VARNUM > 3) order by VARNUM;
quit;


data _long_id;
 set &long;
  by &id;
  retain counter 0;
  if first.&id then counter=1;
  else counter=counter+1;
  if counter=1;
  keep &id;
run;

data _surv_id;
 set &surv;
  keep &id;
run;

data _common_id;
 merge _long_id(IN=IN1) _surv_id(IN=IN2);
 by &id;
 if IN1 and IN2;
run;

proc univariate data=_long_id noprint;
output out=_N_long_id N=Subjects_in_&long;

proc univariate data=_surv_id noprint;
output out=_N_surv_id N=Subjects_in_&surv;

proc univariate data=_common_id noprint;
output out=_N_common_id N=Subjects_Used;

data _N_common_id;
 set _N_common_id;
  call symput('nid', Subjects_Used);
run;

data _N_all_id;
 merge _N_long_id _N_surv_id _N_common_id;
run;

data _&long.2;
 merge &long(IN=IN1) _common_id(IN=IN2);
 by &id;
 if IN1 and IN2;
run;

data _&surv.2;
 merge &surv(IN=IN1) _common_id(IN=IN2);
 by &id;
 if IN1 and IN2;
run;

proc univariate data=_&long.2 noprint;
  var &t;
  by &id;
 output out=_out_t N=N max=tmax;
run;

data _&long.2;
 merge _&long.2 _out_t;
 by &id;
 datalong=1;
run;

data _death;
 set _&surv.2;
  if &delta=1;
run;

%if &npieces = 1 %then %do;
data _&surv.2;
 set _&surv.2;
 new_&id=_N_;
run;
%end;

%if &npieces > 1 %then %do;

 %if &partition = 1 %then %do;
  data _pct;
    do i=1 to %eval(&npieces-1);
     c=i/&npieces*100; output;
    end;
  run;
 %end;

 %if &partition = 2 %then %do;
  data _jdecomp;
   j=&npieces;
   k=floor(log(j)/log(2));
   m=j-2**k;
  run;

  data _pct;
   set _jdecomp;
    do i=1 to 2**k-1;
     c=i/2**k*100; output;
    end;
    do i=1 to m;
     c=(2*i-1)/2**(k+1)*100; output;
    end;
  run;

  proc sort data=_pct;
   by c;
  run;
 %end;


 %if &partition = 3 %then %do;
  data _jdecomp;
   j=&npieces;
   k=floor(log(j)/log(2));
   m=j-2**k;
  run;

  data _pct;
   set _jdecomp;
    do i=1 to 2**k-1;
     c=i/2**k*100; output;
    end;
    do i=1 to m;
	 if mod(i,2) ne 0 then do;
      c=(2**k-i)/2**(k+1)*100; output;
	 end;
	 if mod(i,2)=0 then do;
      c=(2**k+i-1)/2**(k+1)*100; output;
	 end;
    end;
  run;

  proc sort data=_pct;
   by c;
  run;
 %end;


 %if &partition = 4 %then %do;
  data _jdecomp;
   j=&npieces;
   k=floor(log(j)/log(2));
   m=j-2**k;
  run;

  data _pct;
   set _jdecomp;
    do i=1 to 2**k-1;
     c=i/2**k*100; output;
    end;
    do i=1 to m;
     c=(2**(k+1)-(2*i-1))/2**(k+1)*100; output;
    end;
  run;

  proc sort data=_pct;
   by c;
  run;
 %end;

proc sql noprint;
  select c into :pct separated by " "  from _pct;
run;

proc univariate data=_death noprint;
 var &st;
 output out=_pct_&st pctlpts=&pct pctlpre=P;
run;

proc contents data=_pct_&st noprint out=_pct_name;
run;

proc sql noprint;
  select NAME into :p1-:p%left(%eval(&npieces-1)) 
  from _pct_name order by VARNUM;
run;

data _pct2_&st;
 set _pct_&st;
  tie=0;
  %do i=1 %to %eval(&npieces-2);
   %let j=%eval(&i+1);
     if &&p&i = &&p&j then do;
       tie=tie+1;
	 end;
  %end;
  %do i=1 %to %eval(&npieces-2);
   %do j=%eval(&i+1) %to %eval(&npieces-1);
    %do k=%eval(&j+1) %to %eval(&npieces-1);	  
      if &&p&i = &&p&j and &&p&j < &&p&k then &&p&j=&&p&k; 
    %end;
   %end;
  %end;
  call symput('tie', tie);
run;

data _null_;
  call symput('npieces', trim(left(%eval(&npieces-&tie))));
run;

data _pct_distinct;
 set _pct2_&st;
  %do i=1 %to %eval(&npieces-1);
   keep &&p&i;
  %end;
run;

data _&surv.2;
 set _&surv.2;
 if _n_=1 then set _pct_distinct;
 new_&id=_N_;
run;
%end;

data _joint;
 merge _&long.2 _&surv.2;
 by &id;
run;

data _joint2; 
 set _joint;
  by &id;
  retain counter 0;
  if first.&id then counter=1;
  else counter=counter+1;
  if counter=1;
  &var=&st; datalong=0; &t=.; output;
run;

data _tmax_gt_st;
 set _joint2;
  if tmax > &st;
  keep &id;
run;

%let dsid = %sysfunc (open(_tmax_gt_st));
%let anyobs = %sysfunc (attrn(&dsid,ANY));
%let dsid = %sysfunc (close(&dsid));


data _joint;
 set _joint2 _joint;
  drop counter;
run;

proc sort data=_joint;
 by &id;
run;


/* Obtain initial values */ 
ods listing close;
%if &nlcv=0 %then %do;
proc mixed data=_joint method=ml;
  model &var = &t /S cl DDFM=RES;
  random int &t /type=un sub=&id S G;
  ods output CovParms=_Cov SolutionF=_SolutionF SolutionR=_SolutionR;
run;
%end;

%else %do;
proc mixed data=_joint method=ml;
  model &var = &t &lcvs /S cl DDFM=RES;
  random int &t /type=un sub=&id S G;
  ods output CovParms=_Cov SolutionF=_SolutionF SolutionR=_SolutionR;
run;
%end;
ods listing;

proc transpose data=_Cov out=_Cov2;  
 var Estimate;
 id CovParm;
 idlabel CovParm;
run;

data _Cov2(rename=(UN_1_1_=v00 UN_2_1_=v10 UN_2_2_=v11));
 set _Cov2;
  sigma=Residual**0.5;
  drop _NAME_ Residual;
run;

proc transpose data=_SolutionF out=_F2; 
  var Estimate;
  id effect; 
  idlabel Effect; 
run;

proc transpose data=_SolutionR out=_R2; 
  var Estimate; 
  by Subject; 
  id effect; 
  idlabel Effect; 
run;

data _FR; 
  set _R2;
  if _n_=1 then set _F2(rename=(intercept=theta0 &t=theta1)); 
   new_&id=_N_;
   drop _NAME_ &lcvs;  
run; 

data _surv;
 set _joint;
  if datalong=0;
run;

data _TS_step2; 
  merge _surv _FR; 
  by new_&id; 
  theta_0=intercept+theta0;  
  theta_1=&t+theta1; 
run;


data _init1;  
   %do i=1 %to &nscv;
    alpha&i=-0.1;  
   %end; 
   %do i=1 %to &npieces;
    llambda&i=-1;
   %end;
run;


/* AIC_surv,0 & BIC_surv,0 */
proc nlmixed data=_joint2;
  parms /data=_init1;

  alphai=0;
  %do i=1 %to &nscv;
    alphai=alphai+alpha&i*&&scv&i;
  %end;

  %do i=1 %to &npieces;
    lambda&i=exp(llambda&i);
  %end;

  %if &npieces = 1 %then %do;
       logL = (llambda1+alphai)*&delta 
                -lambda1*exp(alphai)*&var; 
  %end;

  %if &npieces = 2 %then %do;
     if &var <= &p1 then
       logL = (llambda1+alphai)*&delta 
                -lambda1*exp(alphai)*&var; 

     if &var > &p1 then
       logL = (llambda2+alphai)*&delta
                -lambda1*exp(alphai)*&p1
                  -lambda2*exp(alphai)*(&var-&p1);  
  %end;

  %if &npieces > 2 %then %do;
     if &var <= &p1 then
         logL = (llambda1+alphai)*&delta 
                  -lambda1*exp(alphai)*&var; 

     if &var > &p1 and &var <= &p2 then
         logL = (llambda2+alphai)*&delta 
                  -lambda1*exp(alphai)*&p1 
                    -lambda2*exp(alphai)*(&var-&p1); 


    %do i=2 %to %eval(&npieces-2);
       %let j=%eval(&i+1);
       if &var > &&p&i and &var <= &&p&j then do;
          s = -lambda1*exp(alphai)*&p1;
	     %do w=2 %to %eval(&i);
	        %let k=%eval(&w-1);
            s = s - lambda&w*exp(alphai)*(&&p&w-&&p&k);
         %end;
	      logL = s -lambda&j*exp(alphai)*(&var-&&p&i)
                + (llambda&j+alphai)*&delta; 
	   end;
    %end;


    %let m=%eval(&npieces-1);
    if &var > &&p&m then do;
        s = -lambda1*exp(alphai)*&p1;
       %do w=2 %to &m;
          %let k=%eval(&w-1);
          s = s - lambda&w*exp(alphai)*(&&p&w-&&p&k);
	   %end;
	    logL = s -lambda&npieces*exp(alphai)*(&var-&&p&m)
               + (llambda&npieces+alphai)*&delta; 
    end;

  %end;

  model &var ~ general(logL);
  ods output ParameterEstimates=_surv_para_est FitStatistics=_surv_fit;
run;

data _surv_fit2;
 set _surv_fit;
 N=_n_;
 if N=2 or N=4;
run;

proc transpose data=_surv_fit2 out=_surv_fit3;
 var value;
 id Descr;
run;

data _surv_fit3(rename=(AIC__smaller_is_better_=AIC_surv0 BIC__smaller_is_better_=BIC_surv0));
 set _surv_fit3;
run;

data _surv_para_est2;
 length Parameter $32;
 set _surv_para_est;
  rename Probt=pValue;
  %do i=1 %to &nscv;
    if Parameter="alpha&i" then Parameter="&&scv&i";
  %end;
  CI='('||trim(left(put(Lower, 6.3)))||', '||trim(left(put(Upper, 6.3)))||')';
run;

proc transpose data=_surv_para_est out=_init2; 
  var Estimate;  
  id Parameter; 
  idlabel Parameter; 
run;

data _init2;
 set _init2;
  beta1=0.1;
  beta2=0.1;
  drop _NAME_;
run;


proc nlmixed data=_TS_step2;
 parms /data=_init2;

  alphai=0;
  %do i=1 %to &nscv;
    alphai=alphai+alpha&i*&&scv&i;
  %end;

  %do i=1 %to &npieces;
    lambda&i=exp(llambda&i);
  %end;

  %if &npieces = 1 %then %do;
       logL = (llambda1+beta1*theta_0+beta2*theta_1+alphai)*&delta 
                -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&var; 
  %end;

  %if &npieces = 2 %then %do;
   if &var <= &p1 then
       logL = (llambda1+beta1*theta_0+beta2*theta_1+alphai)*&delta 
                -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&var; 

   if &var > &p1 then
       logL = (llambda2+beta1*theta_0+beta2*theta_1+alphai)*&delta
                -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&p1
                  -lambda2*exp(beta1*theta_0+beta2*theta_1+alphai)*(&var-&p1);  
  %end;

  %if &npieces > 2 %then %do;
   if &var <= &p1 then
       logL = (llambda1+beta1*theta_0+beta2*theta_1+alphai)*&delta 
                -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&var; 

   if &var > &p1 and &var <= &p2 then
       logL = (llambda2+beta1*theta_0+beta2*theta_1+alphai)*&delta 
                -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&p1 
                  -lambda2*exp(beta1*theta_0+beta2*theta_1+alphai)*(&var-&p1); 


   %do i=2 %to %eval(&npieces-2);
      %let j=%eval(&i+1);
      if &var > &&p&i and &var <= &&p&j then do;
	    s = -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&p1;
	    %do w=2 %to %eval(&i);
	       %let k=%eval(&w-1);
           s = s - lambda&w*exp(beta1*theta_0+beta2*theta_1+alphai)*(&&p&w-&&p&k);
        %end;
	    logL = s -lambda&j*exp(beta1*theta_0+beta2*theta_1+alphai)*(&var-&&p&i)
               + (llambda&j+beta1*theta_0+beta2*theta_1+alphai)*&delta; 
	  end;
   %end;


   %let m=%eval(&npieces-1);
   if &var > &&p&m then do;
     s = -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&p1;
     %do w=2 %to &m;
        %let k=%eval(&w-1);
        s = s - lambda&w*exp(beta1*theta_0+beta2*theta_1+alphai)*(&&p&w-&&p&k);
     %end;
     logL = s -lambda&npieces*exp(beta1*theta_0+beta2*theta_1+alphai)*(&var-&&p&m)
            + (llambda&npieces+beta1*theta_0+beta2*theta_1+alphai)*&delta; 
   end;

  %end;

  model &var ~ general(logL);
  ods output ParameterEstimates=_para_est FitStatistics=_fit;
run;


%let errmess = &syserr;
%if &errmess ^= 0 %then %do;
  %let errtext = &syserrortext;
  %put &errtext;
%end;


%if &ts=1 %then %do;

/* AIC & BIC */
data _fit2;
 set _fit;
  N=_n_;
  if N=1 then Value=-Value/2;
  if N=1 or N=2 or N=4;
run;

proc transpose data=_fit2 out=_fit3;
 var value;
 id Descr;
run;

data _fit3(rename=(_2_Log_Likelihood=Loglik AIC__smaller_is_better_=AIC BIC__smaller_is_better_=BIC));
 set _fit3;
run;


/* HR or estimate for parameter */
data _para_HR;
 length Parameter $32;
 set _para_est;
  %do i=1 %to &nscv;
    if Parameter="alpha&i" then Parameter="HR_&&scv&i";
  %end;
  if Parameter="beta1" then Parameter="HR_beta1";
  if Parameter="beta2" then Parameter="HR_beta2";
  %do i=1 %to &npieces;
    if Parameter="llambda&i" then Parameter="lambda&i";
  %end;
  keep Parameter Estimate Lower Upper;
run;

data _para_HR;
 set _para_HR;
  Estimate=exp(Estimate);
  Lower=exp(Lower);
  Upper=exp(Upper);
  CI='('||trim(left(put(Lower, 6.3)))||', '||trim(left(put(Upper, 6.3)))||')';
 keep Parameter Estimate CI;
run;


data _fit_all;
 merge _fit3 _surv_fit3;
  Delta_AIC=AIC_surv0 - AIC;
  Delta_BIC=BIC_surv0 - BIC;
run;

data _AIC;
 retain Loglik AIC AIC_surv0 Delta_AIC;
 set _fit_all;
  keep Loglik AIC AIC_surv0 Delta_AIC;
run;

data _BIC;
 retain BIC BIC_surv0 Delta_BIC;
 set _fit_all;
  keep BIC BIC_surv0 Delta_BIC;
run;

proc transpose data=_AIC out=_AIC2;
run;

data _AIC2;
 set _AIC2;
  rename _NAME_=AIC_name COL1=AIC_value;
  N=_N_;
run;
  
proc transpose data=_BIC out=_BIC2;
run;

data _BIC2;
 set _BIC2;
  rename _NAME_=BIC_name COL1=BIC_value;
  N=_N_+1;
run;
  
data _AIC_BIC;
 length AIC_name BIC_name $ 20;
 merge _AIC2 _BIC2;
 by N;
run;

data _SolutionF2;
 set _SolutionF;
  rename Probt=pValue;
  rename StdErr=SE;
  CI='('||trim(left(put(Lower, 6.3)))||', '||trim(left(put(Upper, 6.3)))||')';
run;

data _para_est2;
 length Parameter $32;
 set _para_est;
  rename Probt=pValue;
  %do i=1 %to &nscv;
    if Parameter="alpha&i" then Parameter="&&scv&i";
  %end;
  CI='('||trim(left(put(Lower, 6.3)))||', '||trim(left(put(Upper, 6.3)))||')';
run;

proc transpose data=_N_all_id out=_N_all_id2;
run;

data _N_all_id2;
  set _N_all_id2;
   drop _LABEL_;
run;


options ls=256 ps=100 missing="";
ods escapechar="~";
%if &output = %then %do;
ods rtf file="Output for &var under SPM2L-TS with J=&npieces. (Partition=&partition).rtf" startpage=no;
%end;
%if &output ^= %then %do;
ods rtf file="&output..rtf" startpage=no;
%end;
ods listing close;
ods noresults;

proc report data=_N_all_id2 nowd center split="|" headline headskip noheader
   style(report)=[font_face='Times New Roman' font_size=13pt] 
   style(column)=[font_face='Times New Roman' font_size=10pt]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns _NAME_ COL1;

   define  _NAME_/style=[background=GRAYB8];
   define  COL1/display;
   compute before _page_;
     line 'Number of Subjects';
   endcomp;

   title "Output for &var under SPM2L-TS model with J=&npieces (Partition=&partition)"; 
run;

%if &errmess = 0 %then %do;
proc report data=_AIC_BIC nowd center split="|" headline headskip noheader
   style(report) =[font_face='Times New Roman' font_size=13pt]
   style(header) =[font_face='Times New Roman' font_size=10pt just=c foreground=black background=GRAYB8]
   style(column) =[font_face='Times New Roman' font_size=10pt just=c]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns AIC_name AIC_value BIC_name BIC_value;

   define  AIC_name /style=[background=GRAYB8 just=l];
   define  AIC_value /style=[just=r] format=8.2;
   define  BIC_name /style=[background=GRAYB8 just=l];
   define  BIC_value /style=[just=r] format=8.2;

   compute AIC_name;
     if AIC_name="Loglik" then AIC_name="Log Likelihood";
     if AIC_name="AIC_surv0" then AIC_name="AIC~{sub Surv,0}";
     if AIC_name="Delta_AIC" then AIC_name="~{unicode 0394}AIC";
   endcomp;

   compute BIC_name;
     if BIC_name="BIC_surv0" then BIC_name="BIC~{sub Surv,0}";
     if BIC_name="Delta_BIC" then BIC_name="~{unicode 0394}BIC";
   endcomp;

   compute before _page_;
     line 'Fit Statistics';
   endcomp;
run;
%end;

proc report data=_surv_para_est2 nowd center split='|' headline headskip
   style(report) =[font_face='Times New Roman' font_size=13pt] 
   style(column) =[font_face='Times New Roman' font_size=10pt]
   style(header) =[font_face='Times New Roman' font_size=10pt just=r foreground=black background=GRAYB8]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns Parameter Estimate StandardError DF tValue pValue CI Gradient;
   
   define  Parameter/"Param/Var" style=[background=GRAYB8] style(header)=[just=c];
   define  Estimate/display;
   define  StandardError/"SE";
   define  DF/display;
   define  tValue/"T-Value";
   define  pValue/"P-Value";
   define  CI/"95% CI" style=[just=r];
   define  Gradient/display;
   compute Parameter;
	%do i=1 %to &npieces;
    if Parameter="llambda&i" then Parameter="log ~{unicode 03BB}~{sub &i}";
    %end;
   endcomp;
   compute before _page_;
     line 'Survival Parameter Estimates (Survival Alone)';
   endcomp;
run;

ods rtf startpage=now;
%if &errmess ^= 0 %then %do;
 data _err;
  error="ERROR: &errtext";
 run;

proc report data=_err nowd center noheader
   style(report) =[font_face='Times New Roman' font_size=13pt] 
   style(column) =[font_face='Times New Roman' font_size=11pt];

   columns error;
run;
%end;

%if &errmess = 0 %then %do;
proc report data=_SolutionF2 nowd center split='|' headline headskip
   style(report)=[font_face='Times New Roman' font_size=13pt] 
   style(column)=[font_face='Times New Roman' font_size=10pt]
   style(header)=[font_face='Times New Roman' font_size=10pt just=r foreground=black background=GRAYB8]
   style(lines)=[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns Effect Estimate SE DF tValue pValue CI;
   
   define  Effect/"Param/Var" style=[background=GRAYB8] style(header)=[just=c];
   define  Estimate/display;
   define  SE/"SE";
   define  DF/display;
   define  tValue/"T-Value";
   define  pValue/"P-Value";
   define  CI/"95% CI" style=[just=r];
   compute before _page_;
     line 'Longitudinal Parameter Estimates (Stage I)';
   endcomp;
run;


proc report data=_para_est2 nowd center split='|' headline headskip
   style(report)=[font_face='Times New Roman' font_size=13pt] 
   style(column)=[font_face='Times New Roman' font_size=10pt]
   style(header)=[font_face='Times New Roman' font_size=10pt just=r foreground=black background=GRAYB8]
   style(lines)=[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns Parameter Estimate StandardError DF tValue pValue CI Gradient;
   
   define  Parameter/"Param/Var" style=[background=GRAYB8] style(header)=[just=c];
   define  Estimate/display;
   define  StandardError/"SE";
   define  DF/display;
   define  tValue/"T-Value";
   define  pValue/"P-Value";
   define  CI/"95% CI" style=[just=r];
   define  Gradient/display;
   compute Parameter;
    if Parameter="beta1" then Parameter="~{unicode 03B2}~{sub 1}";
    if Parameter="beta2" then Parameter="~{unicode 03B2}~{sub 2}";
	%do i=1 %to &npieces;
    if Parameter="llambda&i" then Parameter="log ~{unicode 03BB}~{sub &i}";
    %end;
   endcomp;
   compute before _page_;
     line 'Survival Parameter Estimates (Stage II)';
   endcomp;
run;

ods rtf startpage=now;
proc report data=_para_HR nowd center split="|" headline headskip
   style(report) =[font_face='Times New Roman' font_size=13pt]
   style(header) =[font_face='Times New Roman' font_size=10pt just=r foreground=black background=GRAYB8]
   style(column) =[font_face='Times New Roman' font_size=10pt]
   style(lines)=[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns Parameter Estimate CI;

   define  Parameter/style=[background=GRAYB8] style(header)=[just=l];
   define  Estimate/display;
   define  CI/"95% CI" style=[just=r];
   compute Parameter;
    if Parameter="HR_beta1" then Parameter="HR_~{unicode 03B2}~{sub 1}";
    if Parameter="HR_beta2" then Parameter="HR_~{unicode 03B2}~{sub 2}";
	%do i=1 %to &npieces;
    if Parameter="lambda&i" then Parameter="~{unicode 03BB}~{sub &i}";
    %end;
   endcomp;
   compute before _page_;
     line 'Hazard Ratios & ~{unicode 03BB} Estimates';
   endcomp;
run;

%if &anyobs=1 %then %do;
data _caution;
 caution='CAUTION: Longitudinal measures are observed after the survival time.';
run;

proc report data=_caution nowd center noheader
   style(report) =[font_face='Times New Roman' font_size=13pt] 
   style(column) =[font_face='Times New Roman' font_size=11pt];

   columns caution;
run;
%end;

%end;

ods rtf close;
ods listing;
ods results;

%end;



%if &ts= or &ts=0 %then %do;

proc transpose data=_para_est out=_initial_surv; 
  var Estimate;  
  id Parameter; 
  idlabel Parameter; 
run;

data _initial_surv;
 set _initial_surv;
  drop _NAME_;
run;

data _F3 (rename=(intercept=theta0 &t=theta1)) ; 
  set _F2;
   %do i=1 %to &nlcv;
    rename &&lcv&i=gamma&i;
   %end;
   drop _NAME_; 
run; 

data _initial;
 merge _Cov2 _F3 _initial_surv;
run;

data _initial;
 set _initial;
  attrib _all_ label='';
run;


%if &initial ^= %then %do;
 data _initial;
  set &initial;
 run;
%end;


/* SPM2L */
proc nlmixed data=_joint &options;
  parms /data=_initial;

  theta_0=theta0+theta0i;
  theta_1=theta1+theta1i;

  gammai=0;
  %do i=1 %to &nlcv;
    gammai=gammai+gamma&i*&&lcv&i;
  %end;

  alphai=0;
  %do i=1 %to &nscv;
    alphai=alphai+alpha&i*&&scv&i;
  %end;

  if datalong=1 then mu = theta_0 + theta_1*&t + gammai;
  if datalong=1 then logL = -0.5*log(2*3.14159)-log(sigma)-0.5*(&var-mu)**2/sigma**2 ;

  %do i=1 %to &npieces;
    lambda&i=exp(llambda&i);
  %end;

  if datalong=0 then do;

    %if &npieces = 1 %then %do;
        logL = (llambda1+beta1*theta_0+beta2*theta_1+alphai)*&delta 
                 -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&var; 
    %end;

    %if &npieces = 2 %then %do;
      if &var <= &p1 then
        logL = (llambda1+beta1*theta_0+beta2*theta_1+alphai)*&delta 
                 -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&var; 

      if &var > &p1 then
        logL = (llambda2+beta1*theta_0+beta2*theta_1+alphai)*&delta
                 -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&p1
                   -lambda2*exp(beta1*theta_0+beta2*theta_1+alphai)*(&var-&p1);  
    %end;

    %if &npieces > 2 %then %do;
     if &var <= &p1 then
         logL = (llambda1+beta1*theta_0+beta2*theta_1+alphai)*&delta 
                  -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&var; 

     if &var > &p1 and &var <= &p2 then
         logL = (llambda2+beta1*theta_0+beta2*theta_1+alphai)*&delta 
                  -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&p1 
                    -lambda2*exp(beta1*theta_0+beta2*theta_1+alphai)*(&var-&p1); 


     %do i=2 %to %eval(&npieces-2);
        %let j=%eval(&i+1);
        if &var > &&p&i and &var <= &&p&j then do;
	       s = -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&p1;
	      %do w=2 %to %eval(&i);
	         %let k=%eval(&w-1);
             s = s - lambda&w*exp(beta1*theta_0+beta2*theta_1+alphai)*(&&p&w-&&p&k);
          %end;
	       logL = s -lambda&j*exp(beta1*theta_0+beta2*theta_1+alphai)*(&var-&&p&i)
                  + (llambda&j+beta1*theta_0+beta2*theta_1+alphai)*&delta; 
	    end;
     %end;


     %let m=%eval(&npieces-1);
     if &var > &&p&m then do;
        s = -lambda1*exp(beta1*theta_0+beta2*theta_1+alphai)*&p1;
       %do w=2 %to &m;
          %let k=%eval(&w-1);
          s = s - lambda&w*exp(beta1*theta_0+beta2*theta_1+alphai)*(&&p&w-&&p&k);
       %end;
        logL = s -lambda&npieces*exp(beta1*theta_0+beta2*theta_1+alphai)*(&var-&&p&m)
               + (llambda&npieces+beta1*theta_0+beta2*theta_1+alphai)*&delta; 
     end;  

    %end;

  end;

  model &var ~ general(logL);
  random theta0i theta1i ~ normal([0,0], [v00, v10, v11]) subject=&id;
  ods output ParameterEstimates=_para_est FitStatistics=_fit;
run;

%let errmess = &syserr;
%if &errmess ^= 0 %then %do;
  %let errtext = &syserrortext;
%end;

/* AIC/BIC decompsition */
data _long;
 set _joint;
  if datalong=0 then delete;
  cons=1;
run;

proc iml;
 use _para_est;
  read all var{Estimate} into mle; 
   sigma = mle[4,];
   theta = mle[5:(6+&nlcv),];
   V = I(2);
   V[1,1] = mle[1, ];
   V[1,2] = mle[2, ];
   V[2,1] = mle[2, ];
   V[2,2] = mle[3, ];

 use _long;
  read all var{new_&id N &var cons &t} into T;
   %do i=1 %to &nlcv;
    read all var{&&lcv&i} into T&i;
	T=T||T&i;
   %end;
   llog = j(&nid,1,0);

  do i=1 to &nid;
   row=loc(T[,1]=i);
   S=T[row,];
   N = S[1,2];
   long = S[,3];
   X = S[,4:(5+&nlcv)];
   W = X[,1:2];
   a = sigma**2*I(N) + W*V*t(W);
   L = det(a)**(-0.5)/(2*3.14159)**(N/2)*exp(-0.5*t(long-X*theta)*inv(a)*(long-X*theta));
   llog[i] = log(L);
  end;

   sum_llog = sum(llog);
   create _sum_llog from sum_llog;
   append from sum_llog;
 quit;

 data _AIC_BIC_long;
  set _sum_llog;
   AIC_long = -2*COL1 + 2*(6+&nlcv);
   BIC_long = -2*COL1 + (6+&nlcv)*log(&nid);
   drop COL1;
 run;

data _fit2;
 set _fit;
  N=_n_;
  if N=1 then Value=-Value/2;
  if N=1 or N=2 or N=4;
run;

proc transpose data=_fit2 out=_fit3;
 var value;
 id Descr;
run;

data _fit3(rename=(_2_Log_Likelihood=Loglik  AIC__smaller_is_better_=AIC BIC__smaller_is_better_=BIC));
 set _fit3;
run;

/* HR or estimate for parameter */
data _para_HR;
 length Parameter $32;
 set _para_est;
  %do i=1 %to &nscv;
    if Parameter="alpha&i" then Parameter="HR_&&scv&i";
  %end;
  if Parameter="beta1" then Parameter="HR_beta1";
  if Parameter="beta2" then Parameter="HR_beta2";
  %do i=1 %to &npieces;
    if Parameter="llambda&i" then Parameter="lambda&i";
  %end;
  keep Parameter Estimate Lower Upper;
run;

data _para_HR;
 set _para_HR;
  if Parameter="HR_beta1" or Parameter="HR_beta2"
  %do i=1 %to &nscv;
    or Parameter="HR_&&scv&i"
  %end;
  %do i=1 %to &npieces;
    or Parameter="lambda&i"
  %end;;
run;


data _para_HR;
 set _para_HR;
  Estimate=exp(Estimate);
  Lower=exp(Lower);
  Upper=exp(Upper);
  CI='('||trim(left(put(Lower, 6.3)))||', '||trim(left(put(Upper, 6.3)))||')';
 keep Parameter Estimate CI;
run;


data _fit_all;
 merge _fit3 _AIC_BIC_long _surv_fit3;
  AIC_con=AIC - AIC_long;
  BIC_con=BIC - BIC_long;
  Delta_AIC=AIC_surv0 - AIC_con;
  Delta_BIC=BIC_surv0 - BIC_con;
run;

data _AIC;
 retain Loglik AIC AIC_long AIC_con AIC_surv0 Delta_AIC;
 set _fit_all;
  keep Loglik AIC AIC_long AIC_con AIC_surv0 Delta_AIC;
run;

data _BIC;
 retain BIC BIC_long BIC_con BIC_surv0 Delta_BIC;
 set _fit_all;
  keep BIC BIC_long BIC_con BIC_surv0 Delta_BIC;
run;

proc transpose data=_AIC out=_AIC2;
run;

data _AIC2;
 set _AIC2;
  rename _NAME_=AIC_name COL1=AIC_value;
  N=_N_;
run;
  
proc transpose data=_BIC out=_BIC2;
run;

data _BIC2;
 set _BIC2;
  rename _NAME_=BIC_name COL1=BIC_value;
  N=_N_+1;
run;
  
data _AIC_BIC;
 length AIC_name BIC_name $ 20;
 merge _AIC2 _BIC2;
 by N;
run;

data _para_est2;
 length Parameter $32;
 set _para_est;
  rename Probt=pValue;
  if Parameter="theta0" then Parameter="Intercept";
  if Parameter="theta1" then Parameter="&t";
  %do i=1 %to &nlcv;
    if Parameter="gamma&i" then Parameter="&&lcv&i";
  %end;
  %do i=1 %to &nscv;
    if Parameter="alpha&i" then Parameter="&&scv&i";
  %end;
  CI='('||trim(left(put(Lower, 6.3)))||', '||trim(left(put(Upper, 6.3)))||')';
run;

data _para_est2;
 set _para_est2;
  if _n_ <= 4 then comp=1;
  if _n_ > 4 and _n_ <= %eval(6+&nlcv) then comp=2;
  if _n_ > %eval(6+&nlcv) then comp=3;
run;

proc transpose data=_N_all_id out=_N_all_id2;
run;

data _N_all_id2;
  set _N_all_id2;
   drop _LABEL_;
run;


options ls=256 ps=100 missing="";
ods escapechar="~";
%if &output = %then %do;
ods rtf file="Output for &var under SPM2L with J=&npieces. (Partition=&partition).rtf" startpage=no;
%end;
%if &output ^= %then %do;
ods rtf file="&output..rtf" startpage=no;
%end;
ods listing close;
ods noresults;

proc report data=_N_all_id2 nowd center split="|" headline headskip noheader
   style(report) =[font_face='Times New Roman' font_size=13pt] 
   style(column) =[font_face='Times New Roman' font_size=10pt]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns _NAME_ COL1;

   define  _NAME_/style=[background=GRAYB8];
   define  COL1/display;
   compute before _page_;
     line 'Number of Subjects';
   endcomp;

   title "Output for &var under SPM2L with J=&npieces (Partition=&partition)"; 
run;


%if &errmess = 0 %then %do;

proc report data=_AIC_BIC nowd center split="|" headline headskip noheader
   style(report) =[font_face='Times New Roman' font_size=13pt]
   style(column) =[font_face='Times New Roman' font_size=10pt just=c]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns AIC_name AIC_value BIC_name BIC_value;

   define  AIC_name /style=[background=GRAYB8 just=l];
   define  AIC_value /style=[just=r] format=8.2;
   define  BIC_name /style=[background=GRAYB8 just=l];
   define  BIC_value /style=[just=r] format=8.2;

   compute AIC_name;
     if AIC_name="Loglik" then AIC_name="Log Likelihood";
     if AIC_name="AIC_long" then AIC_name="AIC~{sub Long}";
     if AIC_name="AIC_con" then AIC_name="AIC~{sub Surv|Long}";
     if AIC_name="AIC_surv0" then AIC_name="AIC~{sub Surv,0}";
     if AIC_name="Delta_AIC" then AIC_name="~{unicode 0394}AIC";
   endcomp;

   compute BIC_name;
     if BIC_name="BIC_long" then BIC_name="BIC~{sub Long}";
     if BIC_name="BIC_con" then BIC_name="BIC~{sub Surv|Long}";
     if BIC_name="BIC_surv0" then BIC_name="BIC~{sub Surv,0}";
     if BIC_name="Delta_BIC" then BIC_name="~{unicode 0394}BIC";
   endcomp;

   compute before _page_;
     line 'Fit Statistics';
   endcomp;
run;
%end;


proc report data=_surv_para_est2 nowd center split='|' headline headskip
   style(report) =[font_face='Times New Roman' font_size=13pt] 
   style(column) =[font_face='Times New Roman' font_size=10pt]
   style(header) =[font_face='Times New Roman' font_size=10pt just=r foreground=black background=GRAYB8]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns Parameter Estimate StandardError DF tValue pValue CI Gradient;
   
   define  Parameter/"Param/Var" style=[background=GRAYB8] style(header)=[just=c];
   define  Estimate/display;
   define  StandardError/"SE";
   define  DF/display;
   define  tValue/"T-Value";
   define  pValue/"P-Value";
   define  CI/"95% CI" style=[just=r];
   define  Gradient/display;
   compute Parameter;
	%do i=1 %to &npieces;
    if Parameter="llambda&i" then Parameter="log ~{unicode 03BB}~{sub &i}";
    %end;
   endcomp;
   compute before _page_;
     line 'Survival Parameter Estimates (Survival Alone)';
   endcomp;
run;

ods rtf startpage=now;

%if &errmess ^= 0 %then %do;
 data _err;
  error="ERROR: &errtext";
 run;

proc report data=_err nowd center noheader
   style(report) =[font_face='Times New Roman' font_size=13pt] 
   style(column) =[font_face='Times New Roman' font_size=11pt];

   columns error;
run;
%end;

%if &errmess = 0 %then %do;

proc report data=_para_est2 nowd center split='|' headline headskip
   style(report) =[font_face='Times New Roman' font_size=13pt] 
   style(column) =[font_face='Times New Roman' font_size=10pt]
   style(header) =[font_face='Times New Roman' font_size=10pt just=r foreground=black background=GRAYB8]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns comp Parameter Estimate StandardError DF tValue pValue CI Gradient;
   
   define  comp/order noprint;
   define  Parameter/"Param/Var" style=[background=GRAYB8] style(header)=[just=c];
   define  Estimate/display;
   define  StandardError/"SE";
   define  DF/display;
   define  tValue/"T-Value";
   define  pValue/"P-Value";
   define  CI/"95% CI" style=[just=r];
   define  Gradient/display;
   compute Parameter;
    if Parameter="v00" then Parameter="~{unicode 03A9}~{sub 00}";
    if Parameter="v10" then Parameter="~{unicode 03A9}~{sub 10}";
    if Parameter="v11" then Parameter="~{unicode 03A9}~{sub 11}";
    if Parameter="sigma" then Parameter="~{unicode 03C3}";
    if Parameter="beta1" then Parameter="~{unicode 03B2}~{sub 1}";
    if Parameter="beta2" then Parameter="~{unicode 03B2}~{sub 2}";
	%do i=1 %to &npieces;
    if Parameter="llambda&i" then Parameter="log ~{unicode 03BB}~{sub &i}";
    %end;
   endcomp;
   compute before _page_;
     line 'Parameter Estimates';
   endcomp;
   compute before comp;
     length text $ 50;
     if comp=1 then do; text='Covariance Parameter Estimates'; len=50; end;
     if comp=2 then do; text='Longitudinal Parameter Estimates'; len=50; end;
	 if comp=3 then do; text='Survival Parameter Estimates'; len=50; end;
     line text $varying. len;
   endcomp;
run;

ods rtf startpage=now;

proc report data=_para_HR nowd center split="|" headline headskip
   style(report) =[font_face='Times New Roman' font_size=13pt]
   style(header) =[font_face='Times New Roman' font_size=10pt just=r foreground=black background=GRAYB8]
   style(column) =[font_face='Times New Roman' font_size=10pt]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns Parameter Estimate CI;

   define  Parameter/style=[background=GRAYB8] style(header)=[just=l];
   define  Estimate/display;
   define  CI/"95% CI" style=[just=r];
   compute Parameter;
    if Parameter="HR_beta1" then Parameter="HR_~{unicode 03B2}~{sub 1}";
    if Parameter="HR_beta2" then Parameter="HR_~{unicode 03B2}~{sub 2}";
	%do i=1 %to &npieces;
    if Parameter="lambda&i" then Parameter="~{unicode 03BB}~{sub &i}";
    %end;
   endcomp;
   compute before _page_;
     line 'Hazard Ratios & ~{unicode 03BB} Estimates';
   endcomp;
run;


%if &anyobs=1 %then %do;
data _caution;
 caution='CAUTION: Longitudinal measures are observed after the survival time.';
run;

proc report data=_caution nowd center noheader
   style(report) =[font_face='Times New Roman' font_size=13pt] 
   style(column) =[font_face='Times New Roman' font_size=11pt];

   columns caution;
run;
%end;


%end;

ods rtf close;
ods listing;
ods results;

%end;

%mend SPM2L;




