/**************************************************************************************************
/* This macro is used to fit TVC model.                                                           *
/*                                                                                                *
/* 1. Variables                                                                                   *
/* id: subject id;                                                                                *
/* var: longitudinal measure;                                                                     *
/* t: time at which the longitudinal measure was taken;                                           *
/* st: survival time;                                                                             *
/* ** Notice (i) t and st have the same unit;                                                     *
/*           (ii) it's better to use month in order to get better convergence;                    *
/* delta: censoring indicator (1=death, 0=censored);                                              *
/* lcv1, lcv2, ...: covariates for the longitudinal component;                                    *
/* scv1, scv2, ...: covariates for the survival component;                                        *
/*                                                                                                *
/* NPIECES: number of intervals for the piecewise constant baseline hazard;                       *
/* PARTITION: partition algorithm (1=ESQP, 2=LBSQP, 3=MBSQP, 4=RBSQP);                            *
/* OPTIONS: options that are available in the PROC NLMIXED statement;                             *
/* OUTPUT: name of the output rich text file (rtf).                                               *
/*                                                                                                *
/* 2. Data sets                                                                                   *
/* LONG: data set for longtudinal measures which may have three columns in this order: id, var,   *
/*     and t, or may have more than these three columns and the rest columns are the covariates   *
/*     for the longitudinal component.                                                            *
/* SURV: data set for survival time whose columns are arranged as follows: id, st, delta, and     *
/*       the rest colums are the covariates.                                                      *
/* INITIAL: data set for the initial values of the parameters in TVC. Not required.               *
/* ** Notice that no missing values are allowed in LONG and SURV.                                 *  
/**************************************************************************************************/

%macro TVC(long, surv, npieces, partition, options, initial, output);
/* Data preparation */
proc contents data=&long noprint out=_long_all;
run;

proc sql noprint;
  select NAME into :id 
     from _long_all where (VARNUM=1);
  select NAME into :var 
     from _long_all where (VARNUM=2);
  select NAME into :t 
     from _long_all where (VARNUM=3);
quit;

%let length=%length(&var);

proc sql noprint;
  select NAME format=$&length.. into :var 
     from _long_all where (VARNUM=2);
quit;

proc contents data=&surv noprint out=_surv_all;
run;

proc sql noprint;
  select NAME into :st
    from _surv_all where (VARNUM=2);
  select NAME into :delta
    from _surv_all where (VARNUM=3);
  select count(*) into :nscv 
    from _surv_all where (VARNUM > 3);
  select NAME into :scv1-:scv%left(&nscv) 
    from _surv_all where (VARNUM > 3) order by VARNUM;
quit;


data _long_id;
 set &long;
  by &id;
  retain counter 0;
  if first.&id then counter=1;
  else counter=counter+1;
  if counter=1;
  keep &id;
run;

data _surv_id;
 set &surv;
  keep &id;
run;

data _common_id;
 merge _long_id(IN=IN1) _surv_id(IN=IN2);
 by &id;
 if IN1 and IN2;
run;

proc univariate data=_long_id noprint;
output out=_N_long_id N=Subjects_in_&long;

proc univariate data=_surv_id noprint;
output out=_N_surv_id N=Subjects_in_&surv;

proc univariate data=_common_id noprint;
output out=_N_common_id N=Subjects_Used;

data _N_all_id;
 merge _N_long_id _N_surv_id _N_common_id;
run;

data _&long.2;
 merge &long(IN=IN1) _common_id(IN=IN2);
 by &id;
 if IN1 and IN2;
run;

data _&surv.2;
 merge &surv(IN=IN1) _common_id(IN=IN2);
 by &id;
 if IN1 and IN2;
run;

proc univariate data=_&long.2 noprint;
  var &t;
  by &id;
 output out=_out_N N=N;
run;


data _&long.2;
 set _&long.2;
 by &id;
 retain counter 0;
 if first.&id then counter=1;
 else counter=counter+1;
 attrib _all_ label="";
run;


proc transpose data=_&long.2 out=_long_tra prefix=&var; 
  var &var; 
  id counter;
  by &id;
run;


proc transpose data=_&long.2 out=_t_tra prefix=&t; 
  var &t; 
  id counter;
  by &id;
run;


data _&long.3 (drop=_NAME_);
 merge _long_tra _t_tra _out_N;
 by &id;
run;

data _death;
 set _&surv.2;
  if &delta=1;
run;

%if &npieces = 1 %then %do;
data _&surv.2;
 set _&surv.2;
 new_&id=_N_;
run;
%end;

%if &npieces > 1 %then %do;

 %if &partition = 1 %then %do;
  data _pct;
    do i=1 to %eval(&npieces-1);
     c=i/&npieces*100; output;
    end;
  run;
 %end;

 %if &partition = 2 %then %do;
  data _jdecomp;
   j=&npieces;
   k=floor(log(j)/log(2));
   m=j-2**k;
  run;

  data _pct;
   set _jdecomp;
    do i=1 to 2**k-1;
     c=i/2**k*100; output;
    end;
    do i=1 to m;
     c=(2*i-1)/2**(k+1)*100; output;
    end;
  run;

  proc sort data=_pct;
   by c;
  run;
 %end;


 %if &partition = 3 %then %do;
  data _jdecomp;
   j=&npieces;
   k=floor(log(j)/log(2));
   m=j-2**k;
  run;

  data _pct;
   set _jdecomp;
    do i=1 to 2**k-1;
     c=i/2**k*100; output;
    end;
    do i=1 to m;
	 if mod(i,2) ne 0 then do;
      c=(2**k-i)/2**(k+1)*100; output;
	 end;
	 if mod(i,2)=0 then do;
      c=(2**k+i-1)/2**(k+1)*100; output;
	 end;
    end;
  run;

  proc sort data=_pct;
   by c;
  run;
 %end;


 %if &partition = 4 %then %do;
  data _jdecomp;
   j=&npieces;
   k=floor(log(j)/log(2));
   m=j-2**k;
  run;

  data _pct;
   set _jdecomp;
    do i=1 to 2**k-1;
     c=i/2**k*100; output;
    end;
    do i=1 to m;
     c=(2**(k+1)-(2*i-1))/2**(k+1)*100; output;
    end;
  run;

  proc sort data=_pct;
   by c;
  run;
 %end;


proc sql noprint;
  select c into :pct separated by " "  from _pct;
run;

proc univariate data=_death noprint;
 var &st;
 output out=_pct_&st pctlpts=&pct pctlpre=P;
run;

proc contents data=_pct_&st noprint out=_pct_name;
run;

proc sql noprint;
  select NAME into :p1-:p%left(%eval(&npieces-1)) 
  from _pct_name order by VARNUM;
run;

data _pct2_&st;
 set _pct_&st;
  tie=0;
  %do i=1 %to %eval(&npieces-2);
   %let j=%eval(&i+1);
     if &&p&i = &&p&j then do;
       tie=tie+1;
	 end;
  %end;
  %do i=1 %to %eval(&npieces-2);
   %do j=%eval(&i+1) %to %eval(&npieces-1);
    %do k=%eval(&j+1) %to %eval(&npieces-1);	  
      if &&p&i = &&p&j and &&p&j < &&p&k then &&p&j=&&p&k; 
    %end;
   %end;
  %end;
  call symput('tie', tie);
run;

data _null_;
  call symput('npieces', trim(left(%eval(&npieces-&tie))));
run;

data _pct_distinct;
 set _pct2_&st;
  %do i=1 %to %eval(&npieces-1);
   keep &&p&i;
  %end;
run;

data _&surv.2;
 set _&surv.2;
 if _n_=1 then set _pct_distinct;
 new_&id=_N_;
run;
%end;


data _joint;
 merge _&long.3 _&surv.2;
 by &id;
run;

proc contents data=_t_tra noprint out=_t_name;
run;

proc contents data=_long_tra noprint out=_long_name;
run;

proc sql noprint;
  select NAME into :tname separated by " " 
    from _t_name where (VARNUM > 2) order by VARNUM;
  select NAME into :longname separated by " " 
    from _long_name where (VARNUM > 2) order by VARNUM;
quit;



data _init1;  
   %do i=1 %to &nscv;
    alpha&i=-0.1;  
   %end; 
   %do i=1 %to &npieces;
    llambda&i=-1;
   %end;
run;


/* AIC_surv,0 & BIC_surv,0 */
proc nlmixed data=_joint;
  parms /data=_init1;

  alphai=0;
  %do i=1 %to &nscv;
    alphai=alphai+alpha&i*&&scv&i;
  %end;

  %do i=1 %to &npieces;
    lambda&i=exp(llambda&i);
  %end;

  %if &npieces = 1 %then %do;
       logL = (llambda1+alphai)*&delta 
                -lambda1*exp(alphai)*&st; 
  %end;

  %if &npieces = 2 %then %do;
     if &st <= &p1 then
       logL = (llambda1+alphai)*&delta 
                -lambda1*exp(alphai)*&st; 

     if &st > &p1 then
       logL = (llambda2+alphai)*&delta
                -lambda1*exp(alphai)*&p1
                  -lambda2*exp(alphai)*(&st-&p1);  
  %end;

  %if &npieces > 2 %then %do;
     if &st <= &p1 then
         logL = (llambda1+alphai)*&delta 
                  -lambda1*exp(alphai)*&st; 

     if &st > &p1 and &st <= &p2 then
         logL = (llambda2+alphai)*&delta 
                  -lambda1*exp(alphai)*&p1 
                    -lambda2*exp(alphai)*(&st-&p1); 


    %do i=2 %to %eval(&npieces-2);
       %let j=%eval(&i+1);
       if &st > &&p&i and &st <= &&p&j then do;
          s = -lambda1*exp(alphai)*&p1;
	     %do w=2 %to %eval(&i);
	        %let k=%eval(&w-1);
            s = s - lambda&w*exp(alphai)*(&&p&w-&&p&k);
         %end;
	      logL = s -lambda&j*exp(alphai)*(&st-&&p&i)
                    + (llambda&j+alphai)*&delta; 
	   end;
    %end;


    %let m=%eval(&npieces-1);
    if &st > &&p&m then do;
        s = -lambda1*exp(alphai)*&p1;
       %do w=2 %to &m;
          %let k=%eval(&w-1);
          s = s - lambda&w*exp(alphai)*(&&p&w-&&p&k);
	   %end;
	    logL = s -lambda&npieces*exp(alphai)*(&st-&&p&m)
                  + (llambda&npieces+alphai)*&delta; 
    end;

  %end;

  model &st ~ general(logL);
  ods output ParameterEstimates=_surv_para_est FitStatistics=_surv_fit;
run;

data _surv_fit2;
 set _surv_fit;
 N=_n_;
 if N=2 or N=4;
run;

proc transpose data=_surv_fit2 out=_surv_fit3;
 var value;
 id Descr;
run;

data _surv_fit3(rename=(AIC__smaller_is_better_=AIC_surv0 BIC__smaller_is_better_=BIC_surv0));
 set _surv_fit3;
run;

data _surv_para_est2;
 length Parameter $32;
 set _surv_para_est;
  rename Probt=pValue;
  %do i=1 %to &nscv;
    if Parameter="alpha&i" then Parameter="&&scv&i";
  %end;
  CI='('||trim(left(put(Lower, 6.3)))||', '||trim(left(put(Upper, 6.3)))||')';
run;

proc transpose data=_surv_para_est out=_initial; 
  var Estimate;  
  id Parameter; 
  idlabel Parameter; 
run;

data _initial;
 set _initial;
  beta=0.1;
  drop _NAME_;
run;


%if &initial ^= %then %do;
 data _initial;
  set &initial;
 run;
%end;



/* TVC */
proc nlmixed data=_joint &options;
  parms /data=_initial;

   array tt{*}  &tname tadd;
   array long{*} &longname varadd;

   alphai=0;
   %do i=1 %to &nscv;
     alphai=alphai+alpha&i*&&scv&i;
   %end;

   %do i=1 %to &npieces;
     lambda&i=exp(llambda&i);
   %end;

   tt1 = 0;
   tt2 = 0;
   logL = 0;

   if tt[1] ne 0 then do;
    ttnewa=tt[1]; 
    longnewa=long[1]; 
    tt[1] = 0; 
    do _i_=1 to N;
	 ttnewb=tt[_i_+1]; longnewb=long[_i_+1];
	 tt[_i_+1]=ttnewa; long[_i_+1]=longnewa;
	 ttnewa=ttnewb;    longnewa=longnewb;
	end;
	N=N+1;
   end;


  %if &npieces = 1 %then %do;
   do _i_= 1 to N;
     if ( tt[_i_] < &st ) then do;
       if ( _i_ < N ) then do;
	     if ( &st < tt[_i_+1] ) then do;
           tt2 = &st; &var = long[_i_]; 
           logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1)
                         + (llambda1 + beta*&var+alphai)*&delta;
	     end;

	     if ( &st >= tt[_i_+1] ) then do;
	       tt2 = tt[_i_+1]; &var = long[_i_];
           logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);  
           tt1 = tt2;
	     end;
       end;

	   if ( _i_ = N ) then do;
          tt2 = &st; &var =long[_i_]; 
          logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1)
                        + (llambda1 + beta*&var+alphai)*&delta;
	   end;
     end;
   end;
  %end;

  %if &npieces = 2 %then %do;
   if &st <= &p1 then do;
     do _i_= 1 to N;
       if ( tt[_i_] < &st ) then do;
         if ( _i_ < N ) then do;
	       if ( &st <= tt[_i_+1] ) then do;
             tt2 = &st; &var = long[_i_]; 
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1)
                           + (llambda1 + beta*&var+alphai)*&delta;
	       end;

	       if ( &st > tt[_i_+1] ) then do;
	         tt2 = tt[_i_+1]; &var = long[_i_];
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);  
             tt1 = tt2;
	       end;
         end;

	     if ( _i_ = N) then do;
             tt2 = &st; &var = long[_i_]; 
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1)
                           + (llambda1 + beta*&var+alphai)*&delta;
	     end;
       end;
     end;
   end;


   if &st > &p1 then do;
     do _i_= 1 to N;
       if ( tt[_i_] < &p1 ) then do;
         if ( _i_ < N ) then do;
	       if ( &p1 <= tt[_i_+1] ) then do;
             tt2 = &p1; &var = long[_i_]; 
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);
			 tt1 = tt2;
	       end;

           if ( &p1 > tt[_i_+1] ) then do;
	         tt2 = tt[_i_+1]; &var = long[_i_];
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);  
             tt1 = tt2;
	       end;
         end;

	     if ( _i_ = N) then do;
             tt2 = &p1; &var = long[_i_]; 
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1)
			               - lambda2*exp(beta*&var+alphai)*(&st-tt2)
                             + (llambda2 + beta*&var+alphai)*&delta;
	     end;
       end;


       if ( tt[_i_] >= &p1 ) and ( tt[_i_] < &st ) then do;
	       if ( tt[_i_-1] < &p1 ) then do;
	          tt2 = tt[_i_]; &var = long[_i_-1];
              logL = logL - lambda2*exp(beta*&var+alphai)*(tt2-tt1);  
              tt1 = tt2;
	       end;

           if ( _i_ < N ) then do;
             if ( &st > tt[_i_+1] ) then do;
	           tt2 = tt[_i_+1]; &var = long[_i_];
               logL = logL - lambda2*exp(beta*&var+alphai)*(tt2-tt1);  
               tt1 = tt2;
	         end;

             if ( &st <= tt[_i_+1] ) then do;
	           tt2 = &st; &var = long[_i_];
               logL = logL - lambda2*exp(beta*&var+alphai)*(tt2-tt1)  
                             + (llambda2 + beta*&var+alphai)*&delta;
	         end;
		   end;

	       if ( _i_ = N ) then do;
               tt2 = &st; &var = long[_i_]; 
               logL = logL - lambda2*exp(beta*&var+alphai)*(tt2-tt1)
                             + (llambda2 + beta*&var+alphai)*&delta;
	       end;
	   end;

       if ( tt[_i_] >= &st ) then do;
	     if ( tt[_i_-1] <= &p1 ) then do;
           tt2 = &st; &var = long[_i_-1]; 
           logL = logL - lambda2*exp(beta*&var+alphai)*(tt2-tt1)
                         + (llambda2 + beta*&var+alphai)*&delta;
		 end;
	   end;

     end;
   end;
  %end;

  %if &npieces > 2 %then %do;
   if &st <= &p1 then do;
     do _i_= 1 to N;
       if ( tt[_i_] < &st ) then do;
         if ( _i_ < N ) then do;
	       if ( &st <= tt[_i_+1] ) then do;
             tt2 = &st; &var = long[_i_]; 
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1)
                           + (llambda1 + beta*&var+alphai)*&delta;
	       end;

	       if ( &st > tt[_i_+1] ) then do;
	         tt2 = tt[_i_+1]; &var = long[_i_];
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);  
             tt1 = tt2;
	       end;
         end;

	     if ( _i_ = N) then do;
             tt2 = &st; &var = long[_i_]; 
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1)
                           + (llambda1 + beta*&var+alphai)*&delta;
	     end;
       end;
     end;
   end;

   %do a=1 %to %eval(&npieces-2);
     %let b=%eval(&a+1);
       if &&p&a < &st and &st <= &&p&b then do;
         do _i_= 1 to N;
           if ( tt[_i_] < &p1 ) then do;
             if ( _i_ < N ) then do;
	           if ( &p1 <= tt[_i_+1] ) then do;
                 tt2 = &p1; &var = long[_i_]; 
                 logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);
		  	     tt1 = tt2;
	           end;

               if ( &p1 > tt[_i_+1] ) then do;
	             tt2 = tt[_i_+1]; &var = long[_i_];
                 logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);  
                 tt1 = tt2;
	           end;
             end;

	         if ( _i_ = N) then do;
                  tt2 = &p1; &var = long[_i_]; 
                  logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);
		        %do w=2 %to %eval(&a);
	              %let k=%eval(&w-1);
			        logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	%end;
			      logL = logL - lambda&b*exp(beta*&var+alphai)*(&st-&&p&a)
                                + (llambda&b + beta*&var+alphai)*&delta;
	         end;
           end;

          %do c=1 %to %eval(&a-1);
           %let d=%eval(&c+1);
           if ( tt[_i_] >= &&p&c ) and ( tt[_i_] < &&p&d ) then do;
	           if ( tt[_i_-1] <= &p1 ) then do;
	               tt2 = tt[_i_]; &var = long[_i_-1];
		         %do w=2 %to %eval(&c);
	               %let k=%eval(&w-1);
			         logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	 %end;
                   logL = logL - lambda&d*exp(beta*&var+alphai)*(tt2-&&p&c);  
                   tt1 = tt2;
	           end;

               %do e=1 %to %eval(&c-1);
                 %let f=%eval(&e+1);
	               if ( tt[_i_-1] > &&p&e ) and ( tt[_i_-1] <= &&p&f )then do;
	                     tt2 = tt[_i_]; &var = long[_i_-1];
		               %do w=%eval(&f+1) %to %eval(&c);
	                     %let k=%eval(&w-1);
			               logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	       %end;
                         logL = logL - lambda&d*exp(beta*&var+alphai)*(tt2-&&p&c);
                         tt1 = tt2;
	               end;
			   %end;

               if ( _i_ < N ) then do;
                 if ( &&p&d > tt[_i_+1] ) then do;
	               tt2 = tt[_i_+1]; &var = long[_i_];
                   logL = logL - lambda&d*exp(beta*&var+alphai)*(tt2-tt1);  
                   tt1 = tt2;
	             end;

                 if ( &&p&d <= tt[_i_+1] ) then do;
	               tt2 = &&p&d; &var = long[_i_];
                   logL = logL - lambda&d*exp(beta*&var+alphai)*(tt2-tt1);
			       tt1 = tt2;
	             end;
		       end;

	           if ( _i_ = N ) then do;
                   tt2 = &&p&d; &var = long[_i_]; 
                   logL = logL - lambda&d*exp(beta*&var+alphai)*(tt2-tt1);
		          %do w=%eval(&d+1) %to %eval(&a);
	                %let k=%eval(&w-1);
			          logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	  %end;
                     logL = logL - lambda&b*exp(beta*&var+alphai)*(&st-&&p&a)
                                   + (llambda&b + beta*&var+alphai)*&delta;
	           end;
	       end;
		  %end;

           if ( tt[_i_] >= &&p&a ) and ( tt[_i_] < &st ) then do;
	           if ( tt[_i_-1] <= &p1 ) then do;
	               tt2 = tt[_i_]; &var = long[_i_-1];
		         %do w=2 %to %eval(&a);
	               %let k=%eval(&w-1);
			         logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	 %end;
                   logL = logL - lambda&b*exp(beta*&var+alphai)*(tt2-&&p&a);  
                   tt1 = tt2;
	           end;

               %do c=1 %to %eval(&a-1);
                 %let d=%eval(&c+1);
	               if ( tt[_i_-1] > &&p&c ) and ( tt[_i_-1] <= &&p&d )then do;
	                     tt2 = tt[_i_]; &var = long[_i_-1];
		               %do w=%eval(&d+1) %to %eval(&a);
	                     %let k=%eval(&w-1);
			               logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	       %end;
                         logL = logL - lambda&b*exp(beta*&var+alphai)*(tt2-&&p&a);
                         tt1 = tt2;
	               end;
			   %end;


               if ( _i_ < N ) then do;
                 if ( &st > tt[_i_+1] ) then do;
	               tt2 = tt[_i_+1]; &var = long[_i_];
                   logL = logL - lambda&b*exp(beta*&var+alphai)*(tt2-tt1);  
                   tt1 = tt2;
	             end;

                 if ( &st <= tt[_i_+1] ) then do;
	               tt2 = &st; &var = long[_i_];
                   logL = logL - lambda&b*exp(beta*&var+alphai)*(tt2-tt1)  
                                 + (llambda&b + beta*&var+alphai)*&delta;
	             end;
		       end;

	           if ( _i_ = N ) then do;
                   tt2 = &st; &var = long[_i_]; 
                   logL = logL - lambda&b*exp(beta*&var+alphai)*(tt2-tt1)
                                 + (llambda&b + beta*&var+alphai)*&delta;
	           end;
	       end;

           if ( tt[_i_] >= &st ) then do;
	           if ( tt[_i_-1] <= &p1 ) then do;
	               tt2 = &st; &var = long[_i_-1];
		         %do w=2 %to %eval(&a);
	               %let k=%eval(&w-1);
			         logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	 %end;
                   logL = logL - lambda&b*exp(beta*&var+alphai)*(tt2-&&p&a)
                                 + (llambda&b + beta*&var+alphai)*&delta;
	           end;

               %do c=1 %to %eval(&a-1);
                 %let d=%eval(&c+1);
	               if ( tt[_i_-1] > &&p&c ) and ( tt[_i_-1] <= &&p&d )then do;
	                     tt2 = &st; &var = long[_i_-1];
		               %do w=%eval(&d+1) %to %eval(&a);
	                     %let k=%eval(&w-1);
			               logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	       %end;
                         logL = logL - lambda&b*exp(beta*&var+alphai)*(tt2-&&p&a)
                                       + (llambda&b + beta*&var+alphai)*&delta;
	               end;
			   %end;
	       end;

         end;
       end;
   %end;


   %let g=%eval(&npieces-1);
   if &st > &&p&g then do;
     do _i_= 1 to N;
       if ( tt[_i_] < &p1) then do;
         if ( _i_ < N ) then do;
	       if ( &p1 <= tt[_i_+1] ) then do;
             tt2 = &p1; &var = long[_i_]; 
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);
			 tt1 = tt2;
	       end;

           if ( &p1 > tt[_i_+1] ) then do;
	         tt2 = tt[_i_+1]; &var = long[_i_];
             logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);  
             tt1 = tt2;
	       end;
         end;

	     if ( _i_ = N) then do;
              tt2 = &p1; &var = long[_i_]; 
              logL = logL - lambda1*exp(beta*&var+alphai)*(tt2-tt1);
		    %do w=2 %to %eval(&g);
	          %let k=%eval(&w-1);
			    logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		    %end;
			  logL = logL - lambda&npieces*exp(beta*&var+alphai)*(&st-&&p&g)
                            + (llambda&npieces + beta*&var+alphai)*&delta;
	     end;
       end;


       %do c=1 %to %eval(&g-1);
         %let d=%eval(&c+1);
           if ( tt[_i_] >= &&p&c ) and ( tt[_i_] < &&p&d ) then do;
	           if ( tt[_i_-1] <= &p1 ) then do;
	               tt2 = tt[_i_]; &var = long[_i_-1];
		         %do w=2 %to %eval(&c);
	               %let k=%eval(&w-1);
			         logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	 %end;
                   logL = logL - lambda&d*exp(beta*&var+alphai)*(tt2-&&p&c);  
                   tt1 = tt2;
	           end;

               %do e=1 %to %eval(&c-1);
                 %let f=%eval(&e+1);
	               if ( tt[_i_-1] > &&p&e ) and ( tt[_i_-1] <= &&p&f )then do;
	                     tt2 = tt[_i_]; &var = long[_i_-1];
		               %do w=%eval(&f+1) %to %eval(&c);
	                     %let k=%eval(&w-1);
			               logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	       %end;
                         logL = logL - lambda&d*exp(beta*&var+alphai)*(tt2-&&p&c);
                         tt1 = tt2;
	               end;
			   %end;

               if ( _i_ < N ) then do;
                 if ( &&p&d > tt[_i_+1] ) then do;
	               tt2 = tt[_i_+1]; &var = long[_i_];
                   logL = logL - lambda&d*exp(beta*&var+alphai)*(tt2-tt1);  
                   tt1 = tt2;
	             end;

                 if ( &&p&d <= tt[_i_+1] ) then do;
	               tt2 = &&p&d; &var = long[_i_];
                   logL = logL - lambda&d*exp(beta*&var+alphai)*(tt2-tt1);
			       tt1 = tt2;
	             end;
		       end;

	           if ( _i_ = N ) then do;
                   tt2 = &&p&d; &var = long[_i_]; 
                   logL = logL - lambda&d*exp(beta*&var+alphai)*(tt2-tt1);
		          %do w=%eval(&d+1) %to %eval(&g);
	                %let k=%eval(&w-1);
			          logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	  %end;
                     logL = logL - lambda&npieces*exp(beta*&var+alphai)*(&st-&&p&g)
                                   + (llambda&npieces + beta*&var+alphai)*&delta;
	           end;
	       end;
	   %end;


       if ( tt[_i_] >= &&p&g ) and ( tt[_i_] < &st ) then do;
	       if ( tt[_i_-1] <= &p1 ) then do;
	           tt2 = tt[_i_]; &var = long[_i_-1];
		     %do w=2 %to %eval(&g);
	           %let k=%eval(&w-1);
			     logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     %end;
               logL = logL - lambda&npieces*exp(beta*&var+alphai)*(tt2-&&p&g);  
               tt1 = tt2;
	       end;

           %do c=1 %to %eval(&g-1);
             %let d=%eval(&c+1);
	           if ( tt[_i_-1] > &&p&c ) and ( tt[_i_-1] <= &&p&d )then do;
	                 tt2 = tt[_i_]; &var = long[_i_-1];
		           %do w=%eval(&d+1) %to %eval(&g);
	                 %let k=%eval(&w-1);
			           logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	   %end;
                     logL = logL - lambda&npieces*exp(beta*&var+alphai)*(tt2-&&p&g);
                     tt1 = tt2;
	           end;
	       %end;


           if ( _i_ < N ) then do;
             if ( &st > tt[_i_+1] ) then do;
	           tt2 = tt[_i_+1]; &var = long[_i_];
               logL = logL - lambda&npieces*exp(beta*&var+alphai)*(tt2-tt1);  
               tt1 = tt2;
	         end;

             if ( &st <= tt[_i_+1] ) then do;
	           tt2 = &st; &var = long[_i_];
               logL = logL - lambda&npieces*exp(beta*&var+alphai)*(tt2-tt1)  
                             + (llambda&npieces + beta*&var+alphai)*&delta;
	         end;
		   end;

	       if ( _i_ = N ) then do;
               tt2 = &st; &var = long[_i_]; 
               logL = logL - lambda&npieces*exp(beta*&var+alphai)*(tt2-tt1)
                             + (llambda&npieces + beta*&var+alphai)*&delta;
	       end;
	   end;


       if ( tt[_i_] >= &st ) then do;
	       if ( tt[_i_-1] <= &p1 ) then do;
	             tt2 = &st; &var = long[_i_-1];
		       %do w=2 %to %eval(&g);
	             %let k=%eval(&w-1);
			       logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		       %end;
                 logL = logL - lambda&npieces*exp(beta*&var+alphai)*(tt2-&&p&g)
                               + (llambda&npieces + beta*&var+alphai)*&delta;
	       end;

           %do c=1 %to %eval(&g-1);
             %let d=%eval(&c+1);
	           if ( tt[_i_-1] > &&p&c ) and ( tt[_i_-1] <= &&p&d )then do;
	                 tt2 = &st; &var = long[_i_-1];
		           %do w=%eval(&d+1) %to %eval(&g);
	                 %let k=%eval(&w-1);
			           logL = logL - lambda&w*exp(beta*&var+alphai)*(&&p&w-&&p&k);
		     	   %end;
                     logL = logL - lambda&npieces*exp(beta*&var+alphai)*(tt2-&&p&g)
                                   + (llambda&npieces + beta*&var+alphai)*&delta;
	           end;
	       %end;
	   end;

     end;
   end;
  %end;

  model &st ~ general(logL);
  ods output ParameterEstimates=_para_est FitStatistics=_fit;
run;

%let errmess = &syserr;
%if &errmess ^= 0 %then %do;
  %let errtext = &syserrortext;
%end;

/* AIC & BIC */
data _fit2;
 set _fit;
  N=_n_;
  if N=1 then Value=-Value/2;
  if N=1 or N=2 or N=4;
run;

proc transpose data=_fit2 out=_fit3;
 var value;
 id Descr;
run;

data _fit3(rename=(_2_Log_Likelihood=Loglik AIC__smaller_is_better_=AIC BIC__smaller_is_better_=BIC));
 set _fit3;
run;


/* HR or estimate for parameter */
data _para_HR;
 length Parameter $32;
 set _para_est;
  %do i=1 %to &nscv;
    if Parameter="alpha&i" then Parameter="HR_&&scv&i";
  %end;
  if Parameter="beta" then Parameter="HR_beta";
  %do i=1 %to &npieces;
    if Parameter="llambda&i" then Parameter="lambda&i";
  %end;
  keep Parameter Estimate Lower Upper;
run;


data _para_HR;
 set _para_HR;
  Estimate=exp(Estimate);
  Lower=exp(Lower);
  Upper=exp(Upper);
  CI='('||trim(left(put(Lower, 6.3)))||', '||trim(left(put(Upper, 6.3)))||')';
 keep Parameter Estimate CI;
run;

data _fit_all;
 merge _fit3 _surv_fit3;
  Delta_AIC=AIC_surv0 - AIC;
  Delta_BIC=BIC_surv0 - BIC;
run;

data _AIC;
 retain Loglik AIC AIC_surv0 Delta_AIC;
 set _fit_all;
  keep Loglik AIC AIC_surv0 Delta_AIC;
run;

data _BIC;
 retain BIC BIC_surv0 Delta_BIC;
 set _fit_all;
  keep BIC BIC_surv0 Delta_BIC;
run;

proc transpose data=_AIC out=_AIC2;
run;

data _AIC2;
 set _AIC2;
  rename _NAME_=AIC_name COL1=AIC_value;
  N=_N_;
run;
  
proc transpose data=_BIC out=_BIC2;
run;

data _BIC2;
 set _BIC2;
  rename _NAME_=BIC_name COL1=BIC_value;
  N=_N_+1;
run;
  
data _AIC_BIC;
 length AIC_name BIC_name $ 20;
 merge _AIC2 _BIC2;
 by N;
run;


data _para_est2;
 length Parameter $32;
 set _para_est;
  rename Probt=pValue;
  %do i=1 %to &nscv;
    if Parameter="alpha&i" then Parameter="&&scv&i";
  %end;
  CI='('||trim(left(put(Lower, 6.3)))||', '||trim(left(put(Upper, 6.3)))||')';
run;

proc transpose data=_N_all_id out=_N_all_id2;
run;

data _N_all_id2;
  set _N_all_id2;
   drop _LABEL_;
run;



options ls=256 ps=100 missing="";
ods escapechar="~";
%if &output = %then %do;
ods rtf file="Output for &var under TVC with J=&npieces. (Partition=&partition).rtf" startpage=no;
%end;
%if &output ^= %then %do;
ods rtf file="&output..rtf" startpage=no;
%end;
ods listing close;
ods noresults;

proc report data=_N_all_id2 nowd center split="|" headline headskip noheader
   style(report)=[font_face='Times New Roman' font_size=13pt] 
   style(column)=[font_face='Times New Roman' font_size=10pt]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns _NAME_ COL1;

   define  _NAME_/style=[background=GRAYB8];
   define  COL1/display;
   compute before _page_;
     line 'Number of Subjects';
   endcomp;

   title "Output for &var under TVC model with J=&npieces (Partition=&partition)"; 
 
run;

%if &errmess = 0 %then %do;
proc report data=_AIC_BIC nowd center split="|" headline headskip noheader
   style(report) =[font_face='Times New Roman' font_size=13pt]
   style(column) =[font_face='Times New Roman' font_size=10pt just=c]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns AIC_name AIC_value BIC_name BIC_value;

   define  AIC_name /style=[background=GRAYB8 just=l];
   define  AIC_value /style=[just=r] format=8.2;
   define  BIC_name /style=[background=GRAYB8 just=l];
   define  BIC_value /style=[just=r] format=8.2;

   compute AIC_name;
     if AIC_name="Loglik" then AIC_name="Log Likelihood";
     if AIC_name="AIC_surv0" then AIC_name="AIC~{sub Surv,0}";
     if AIC_name="Delta_AIC" then AIC_name="~{unicode 0394}AIC";
   endcomp;

   compute BIC_name;
     if BIC_name="BIC_surv0" then BIC_name="BIC~{sub Surv,0}";
     if BIC_name="Delta_BIC" then BIC_name="~{unicode 0394}BIC";
   endcomp;

   compute before _page_;
     line 'Fit Statistics';
   endcomp;
run;
%end;


proc report data=_surv_para_est2 nowd center split='|' headline headskip
   style(report) =[font_face='Times New Roman' font_size=13pt] 
   style(column) =[font_face='Times New Roman' font_size=10pt]
   style(header) =[font_face='Times New Roman' font_size=10pt just=r foreground=black background=GRAYB8]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns Parameter Estimate StandardError DF tValue pValue CI Gradient;
   
   define  Parameter/"Param/Var" style=[background=GRAYB8] style(header)=[just=c];
   define  Estimate/display;
   define  StandardError/"SE";
   define  DF/display;
   define  tValue/"T-Value";
   define  pValue/"P-Value";
   define  CI/"95% CI" style=[just=r];
   define  Gradient/display;
   compute Parameter;
	%do i=1 %to &npieces;
    if Parameter="llambda&i" then Parameter="log ~{unicode 03BB}~{sub &i}";
    %end;
   endcomp;
   compute before _page_;
     line 'Survival Parameter Estimates (Survival Alone)';
   endcomp;
run;

ods rtf startpage=now;
%if &errmess ^= 0 %then %do;
 data _err;
  error="ERROR: &errtext";
 run;

proc report data=_err nowd center split="|" headline headskip
   style(report) =[font_face='Times New Roman' font_size=13pt] 
   style(column) =[font_face='Times New Roman' font_size=10pt]
   style(header) =[font_face='Times New Roman' font_size=11pt just=c foreground=black background=GRAYB8]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns error;
   define  error /"Error Message for TVC";
run;
%end;

%if &errmess = 0 %then %do;
proc report data=_para_est2 nowd center split='|' headline headskip
   style(report)=[font_face='Times New Roman' font_size=13pt] 
   style(column)=[font_face='Times New Roman' font_size=10pt]
   style(header)=[font_face='Times New Roman' font_size=10pt just=r foreground=black background=GRAYB8]
   style(lines)=[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns Parameter Estimate StandardError DF tValue pValue CI Gradient;
   
   define  Parameter/"Param/Var" style=[background=GRAYB8] style(header)=[just=c];
   define  Estimate/display;
   define  StandardError/"SE";
   define  DF/display;
   define  tValue/"T-Value";
   define  pValue/"P-Value";
   define  CI/"95% CI" style=[just=r];
   define  Gradient/display;
   compute Parameter;
    if Parameter="beta" then Parameter="~{unicode 03B2}";
	%do i=1 %to &npieces;
    if Parameter="llambda&i" then Parameter="log ~{unicode 03BB}~{sub &i}";
    %end;
   endcomp;
   compute before _page_;
     line 'Parameter Estimates';
   endcomp;
run;

proc report data=_para_HR nowd center split="|" headline headskip
   style(report) =[font_face='Times New Roman' font_size=13pt]
   style(header) =[font_face='Times New Roman' font_size=10pt just=r foreground=black background=GRAYB8]
   style(column) =[font_face='Times New Roman' font_size=10pt]
   style(lines) =[font_face='Times New Roman' font_size=11pt just=c font_weight=bold foreground=black background=GRAYB8];

   columns Parameter Estimate CI;

   define  Parameter/style=[background=GRAYB8] style(header)=[just=l];
   define  Estimate/display;
   define  CI/"95% CI" style=[just=r];
   compute Parameter;
    if Parameter="HR_beta" then Parameter="HR_~{unicode 03B2}";
	%do i=1 %to &npieces;
    if Parameter="lambda&i" then Parameter="~{unicode 03BB}~{sub &i}";
    %end;
   endcomp;
   compute before _page_;
     line 'Hazard Ratios & ~{unicode 03BB} Estimates';
   endcomp;
run;
%end;

ods rtf close;
ods listing;
ods results;

%mend TVC;

